Package: litedown
Type: Package
Title: A Lightweight Version of R Markdown
Version: 0.11
Authors@R: c(
    person("Yihui", "Xie", role = c("aut", "cre"), email = "xie@yihui.name", comment = c(ORCID = "0000-0003-0645-5666", URL = "https://yihui.org")),
    person("Tim", "Taylor", role = "ctb", comment = c(ORCID = "0000-0002-8587-7113")),
    person()
    )
Description: Render R Markdown to Markdown (without using 'knitr'), and Markdown
    to lightweight HTML or 'LaTeX' documents with the 'commonmark' package (instead
    of 'Pandoc'). Some missing Markdown features in 'commonmark' are also
    supported, such as raw HTML or 'LaTeX' blocks, 'LaTeX' math, superscripts,
    subscripts, footnotes, element attributes, and appendices,
    but not all 'Pandoc' Markdown features are (or will be) supported. With
    additional JavaScript and CSS, you can also create HTML slides and articles.
    This package can be viewed as a trimmed-down version of R Markdown and
    'knitr'. It does not aim at rich Markdown features or a large variety of
    output formats (the primary formats are HTML and 'LaTeX'). Book and website
    projects of multiple input documents are also supported.
Depends: R (>= 3.2.0)
Imports: utils, commonmark (>= 2.0.0), xfun (>= 0.55)
Suggests: rbibutils, rstudioapi, testit, tinyimg, tinytex
License: MIT + file LICENSE
URL: https://github.com/yihui/litedown
BugReports: https://github.com/yihui/litedown/issues
VignetteBuilder: litedown
Encoding: UTF-8
Config/roxygen2/version: 8.1.0
NeedsCompilation: no
Packaged: 2026-08-26 16:20:25 UTC; yihui
Author: Yihui Xie [aut, cre] (ORCID: <https://orcid.org/0000-0003-0645-5666>,
    URL: https://yihui.org),
  Tim Taylor [ctb] (ORCID: <https://orcid.org/0000-0002-8587-7113>)
Maintainer: Yihui Xie <xie@yihui.name>
Repository: CRAN
Date/Publication: 2026-08-26 17:00:10 UTC
