# Generated by roxygen2: do not edit by hand

S3method(print,litedown_env)
S3method(record_print,data.frame)
S3method(record_print,knitr_kable)
S3method(record_print,matrix)
S3method(record_print,tbl_df)
export(crack)
export(engines)
export(fiss)
export(fuse)
export(fuse_book)
export(fuse_env)
export(fuse_site)
export(get_context)
export(html_format)
export(latex_format)
export(mark)
export(markdown_options)
export(pkg_citation)
export(pkg_code)
export(pkg_desc)
export(pkg_manual)
export(pkg_news)
export(raw_text)
export(reactor)
export(roam)
export(sieve)
export(timing_data)
export(vest)
import(utils)
importFrom(xfun,Rscript_call)
importFrom(xfun,alnum_id)
importFrom(xfun,base64_uri)
importFrom(xfun,csv_options)
importFrom(xfun,del_empty_dir)
importFrom(xfun,dir_create)
importFrom(xfun,divide_chunk)
importFrom(xfun,download_cache)
importFrom(xfun,exit_call)
importFrom(xfun,fenced_block)
importFrom(xfun,fenced_div)
importFrom(xfun,file_exists)
importFrom(xfun,file_ext)
importFrom(xfun,grep_sub)
importFrom(xfun,html_escape)
importFrom(xfun,html_tag)
importFrom(xfun,html_value)
importFrom(xfun,in_dir)
importFrom(xfun,is_abs_path)
importFrom(xfun,is_blank)
importFrom(xfun,is_rel_path)
importFrom(xfun,loadable)
importFrom(xfun,mime_type)
importFrom(xfun,new_app)
importFrom(xfun,new_record)
importFrom(xfun,normalize_path)
importFrom(xfun,parse_only)
importFrom(xfun,prose_index)
importFrom(xfun,raw_string)
importFrom(xfun,read_all)
importFrom(xfun,read_utf8)
importFrom(xfun,record_print)
importFrom(xfun,relative_path)
importFrom(xfun,same_path)
importFrom(xfun,sans_ext)
importFrom(xfun,set_envvar)
importFrom(xfun,split_lines)
importFrom(xfun,try_error)
importFrom(xfun,try_silent)
importFrom(xfun,with_ext)
importFrom(xfun,write_utf8)
