---
title: Raw Output from Markdown Examples
output:
  litedown::html_format:
    meta:
      css: ["@default"]
vignette: >
  %\VignetteEngine{litedown::vignette}
  %\VignetteIndexEntry{Some Markdown Examples}
---

This vignette shows some examples for different Markdown rendering options.

```{r}
#| render-options, tidy=FALSE,
#| file=litedown:::pkg_file('examples', 'render-options.R'), comment=''
```
