---
title: HTML Output from Markdown Examples
output:
  litedown::html_format:
    meta:
      css: ["@default"]
vignette: >
  %\VignetteEngine{litedown::vignette}
  %\VignetteIndexEntry{HTML Output from Markdown Examples}
---

This vignette shows how the output looks like in HTML from the other vignette `markdown-examples.Rmd`, in which the literal HTML code was shown.

```{r}
#| markdown-output, results='asis',
#| file=litedown:::pkg_file('examples', 'render-options.R')
```

```{r}
#| smartypants, results='asis', eval=isTRUE(l10n_info()[['UTF-8']])

# smartypants example
cat(mark("1/2 (c)"))
cat(mark("1/2 (c)", options = "+smartypants"))

mkd <- paste(names(litedown:::pants), collapse = ' ')
cat(mark(mkd, options = "+smartypants"))
```
