# Technical Notes {#chp:notes}

## Embedding resources

When `https` resources needs to be embedded (via the `embed_resources` option),
only these elements are considered:

``` html
<img src="..." />
<link rel="stylesheet" href="...">
<script src="..."></script>
```

Background images set in the attribute `style="background-image: url(...)"` are
also considered. If an external CSS file contains `url()` resources, these
resources will also be downloaded and embedded.

## CSS for margin content

It's quite simple to move an element into the margin using CSS. For example, the
`.side-right` class in this article is roughly defined as:

``` css
.side-right {
  width: 200px;
  float: right;
  margin-right: -200px
}
```

That basically means the width of the element is 200px and it floats to the
right. Now its right side will touch the right margin of its parent element (the
article body). What we need to do next is move it further to the right by 200px
(i.e., its width), which is done by the `-200px` right margin. Remember, a
positive right margin in CSS moves an element to the left, and a negative right
margin moves it to the right.
