% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fuse.R
\name{get_context}
\alias{get_context}
\title{Get the \code{fuse()} context}
\usage{
get_context(item = NULL)
}
\arguments{
\item{item}{The name of the context item.}
}
\value{
If the \code{item} is provided, return its value in the context. If
\code{NULL}, the whole context (an environment) is returned.
}
\description{
A helper function to query the \code{\link[=fuse]{fuse()}} context (such as the input file path
or the output format name) when called inside a code chunk.
}
\examples{
litedown::get_context("input")
litedown::get_context("format")
names(litedown::get_context())  # all available items
}
